//////////////////////////////////////////////////////////////////////////////////
// Engineer:       James C. Ahlstrom
// 
// Create Date:    7 December 2009
// Design Name:    Digital receiver and transmitter
// Module Name:    Receiver
// Project Name:   Transceiver
// Target Devices: Cyclone 3
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module Receiver(
	input clock,
	input [15:0] adc_data,
	input eth_nRESET,
	input [31:0] rx_tune_phase,
	input [31:0] rx_tune_phase_2,
	input [31:0] tx_tune_phase,
	output [31:0] vna_tune_phase,
	input [7:0] rx_ctrl,
	input [7:0] rx_ctrl_2,
	output reg [31:0] mem_data,
	output reg [9:0] mem_write_addr,
	output reg mem_wen,
	output reg mem_block,				// Index of available block 0 or 1
	output reg [31:0] mem_data_2,
	output reg [9:0] mem_write_addr_2,
	output reg mem_wen_2,
	output reg mem_block_2,
	input [15:0] vna_count,				// number of VNA scan frequencies, or zero
	input [15:0] tx_cordic_out_i,
	input [15:0] tx_cordic_out_q,
	input hpsdr_mode,
	input ptt_hpsdr,
   input	dash,
   input	dot,
   input	clipping,
	input two_receiver
	);
   
	wire [15:0] h_mic_data = 16'd0;
	wire data_avail = vna_count != 0 ? vna_avail : decim_avail;
	wire signed [23:0] data_real  = vna_count != 0 ? vna_real  : decim_real;
	wire signed [23:0] data_imag  = vna_count != 0 ? vna_imag  : decim_imag;
	   //			
   wire data_avail_2 = decim_avail_2;
   wire signed [23:0]data_real_2 = decim_real_2; 
	wire signed [23:0]data_imag_2 = decim_imag_2;
	//
	// This is the VNA module.  It runs as an alternative to the above filters.
	wire vna_avail;
	wire signed [23:0] vna_real, vna_imag;
	VNA m_vna (clock, eth_nRESET, adc_data, tx_cordic_out_i, tx_cordic_out_q, vna_count,
		          rx_tune_phase, tx_tune_phase, vna_tune_phase,
		          vna_avail, vna_real, vna_imag);

	// Memory blocks for ADC 1 data
	reg [31:0] mem_data3;		// data to write
	reg [15:0] mem_save;		// odd two bytes to save from 6-byte sample
	reg [7:0] rx_state;		// state machine for receive
	parameter [10:0] MEM_BYTE_SIZE = 11'd1440;
	parameter [8:0]  MEM_DWORD_SIZE = MEM_BYTE_SIZE[10:2];	// divide by 4
	parameter sRx0		= 12;
	parameter sRx		= 0;
	parameter sRx1		= 1;
	parameter sRx2		= 2;
	parameter sRx3		= 3;
	parameter sRx4		= 4;
	parameter sRx5		= 5;
	parameter sRx6		= 6;
	parameter sRx7		= 7;
	parameter sRx8		= 8;
	parameter sRx9		= 9;
	parameter sRx10   = 10;
	parameter sRx11   = 11;
	parameter hRx0		= 15;// HPSDR protocol
	parameter hRx1		= 16;
	parameter hRx2		= 17;
	parameter hRx3		= 18;
	parameter hRx4		= 19;
	parameter hRx5		= 20;
	parameter hRx6		= 21;
	parameter hRx7		= 22;
	parameter hRx8		= 23;
	parameter hRx9		= 24;
	parameter hRx10	= 25;
	parameter hRx11	= 26;
	parameter hRx12	= 27;
	parameter hRx13	= 28;
	parameter hRx13_1	   = 29;
	parameter hRx13_2	   = 30;
	parameter hRx13_3	   = 31;
	parameter hRx13_4  	= 32;
	parameter hRx13_5 	= 33;
	parameter hRx13_6 	= 34;
	parameter hRx13_7	   = 35;
	parameter hRx13_8 	= 36;
	parameter hRx13_9	   = 37;
	parameter hRx13_10	= 38;
	parameter hRx13_11	= 39;
	parameter hRx13_12	= 40;
	parameter hRx13_13	= 41;
	parameter hRx13_14	= 42;
	parameter hRx13_15	= 43;
	parameter hRx14	= 44;
	parameter hRx15	= 45;
	
	// Memory blocks for ADC 2 data
	reg [31:0] mem_data3_2;		// data to write
	reg [15:0] mem_save_2;		// odd two bytes to save from 6-byte sample
	reg [7:0] rx_state_2;		// state machine for receive
	parameter sRx0_2		= 12;
	parameter sRx_2		= 0;
	parameter sRx1_2		= 1;
	parameter sRx2_2		= 2;
	parameter sRx3_2		= 3;
	parameter sRx4_2		= 4;
	parameter sRx5_2		= 5;
	parameter sRx6_2		= 6;
	parameter sRx7_2		= 7;
	parameter sRx8_2		= 8;
	parameter sRx9_2		= 9;
	parameter sRx10_2   = 10;
	parameter sRx11_2   = 11;
	
// Receiver 1
	// Tune ADC data with CORDIC
	wire signed [21:0] cordic_real, cordic_imag;
	cordic_rx rx_cordic (clock, rx_tune_phase, adc_data, cordic_imag, cordic_real);
   // Receive CIC filters followed by FIR filter
   wire decimA_avail, decimA_avail_r;
   wire signed [17:0] decimA_real, decimA_imag;
   //
   CicDecim #(.STAGES(3), .DECIMATION(8), .IN_WIDTH(22), .ACC_WIDTH(31), .OUT_WIDTH(18))      
     cic_i(clock, 1'b1, decimA_avail, cordic_imag, decimA_imag);
   CicDecim #(.STAGES(3), .DECIMATION(8), .IN_WIDTH(22), .ACC_WIDTH(31), .OUT_WIDTH(18))      
     cic_q(clock, 1'b1, decimA_avail_r, cordic_real, decimA_real);
   //  Variable CIC filter - in width = out width = 18 bits, decimation rate = 2 to 40 
   wire decimB_avail, decimB_avail_r;
   wire signed [17:0] decimB_real, decimB_imag;
	wire [5:0] rate = rx_ctrl[5:0] + 1'd1;
   CicVarDecim #(.STAGES(5), .IN_WIDTH(18), .ACC_WIDTH(45), .OUT_WIDTH(18))
      varcic_i(rate, clock, decimA_avail, decimB_avail, decimA_imag, decimB_imag); 
   CicVarDecim #(.STAGES(5), .IN_WIDTH(18), .ACC_WIDTH(45), .OUT_WIDTH(18))
      varcic_q(rate, clock, decimA_avail, decimB_avail_r, decimA_real, decimB_real); 
   //
   wire decim_avail;
   wire signed [23:0] decim_real, decim_imag;	 
	firX8R8 fir (clock, decimB_avail, decimB_real, decimB_imag,
			decim_avail, decim_real, decim_imag);

// Receiver 2
	// Tune ADC data with CORDIC
	wire signed [21:0] cordic_real_2, cordic_imag_2;
	cordic_rx rx_cordic_2 (clock, rx_tune_phase_2, adc_data, cordic_imag_2, cordic_real_2);
   // Receive CIC filters followed by FIR filter
   wire decimA_avail_2, decimA_avail_r_2;
   wire signed [17:0] decimA_real_2, decimA_imag_2;
   //
   CicDecim #(.STAGES(3), .DECIMATION(8), .IN_WIDTH(22), .ACC_WIDTH(31), .OUT_WIDTH(18))      
     cic_i_2(clock, 1'b1, decimA_avail_2, cordic_imag_2, decimA_imag_2);
   CicDecim #(.STAGES(3), .DECIMATION(8), .IN_WIDTH(22), .ACC_WIDTH(31), .OUT_WIDTH(18))      
     cic_q_2(clock, 1'b1, decimA_avail_r_2, cordic_real_2, decimA_real_2);
   //  Variable CIC filter - in width = out width = 18 bits, decimation rate = 2 to 40 
   wire decimB_avail_2, decimB_avail_r_2;
   wire signed [17:0] decimB_real_2, decimB_imag_2;
	wire [5:0] rate_2 = rx_ctrl_2[5:0] + 1'd1;
   CicVarDecim #(.STAGES(5), .IN_WIDTH(18), .ACC_WIDTH(45), .OUT_WIDTH(18))
      varcic_i_2(rate_2, clock, decimA_avail_2, decimB_avail_2, decimA_imag_2, decimB_imag_2); 
   CicVarDecim #(.STAGES(5), .IN_WIDTH(18), .ACC_WIDTH(45), .OUT_WIDTH(18))
      varcic_q_2(rate_2, clock, decimA_avail_2, decimB_avail_r_2, decimA_real_2, decimB_real_2); 
   //
   wire decim_avail_2;
   wire signed [23:0] decim_real_2, decim_imag_2;	 
	firX8R8 fir_2 (clock, decimB_avail_2, decimB_real_2, decimB_imag_2,
			decim_avail_2, decim_real_2, decim_imag_2);

//------------------------------------------------------------------------
//------------------------------------------------------------------------	
    // HPSDR protocol
	reg [7:0] C0, C1, C2, C3, C4;
	reg [2:0] C0_addr_cnt;
	reg [7:0] C0_2, C1_2, C2_2, C3_2, C4_2;
	reg [2:0] C0_addr_cnt_2;
	reg two_receiver_s;
	

	always @(posedge clock)		// Rx state machine
	begin
		if (eth_nRESET == 0)	// we are in reset
		begin
			mem_write_addr <= 1'd0;
			mem_wen <= 0;
			C0_addr_cnt <= 0;
			rx_state <= sRx;
		end
		else case (rx_state)
		sRx:
		begin
		   if (hpsdr_mode) rx_state <= hRx0; else rx_state <= sRx0;
			two_receiver_s <= two_receiver;
		end
		// HiqSDR mode
		sRx0:		// Wait for first sample of two samples
		begin
			if (data_avail)
			begin		// write first dword, save last two bytes LITTLE-ENDIAN
		      mem_data <= {data_imag[7:0], data_real};  
				mem_save <= data_imag[23:8]; 
				rx_state <= sRx1;
			end
		end
		sRx1:
		begin
			mem_wen <= 1'd1;
			rx_state <= sRx2;
		end
		sRx2:
		begin
			mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			rx_state <= sRx3;		// wait for next sample
		end
		sRx3:		// Wait for second sample of two samples
		begin
			if (data_avail)
			begin
				mem_data <= {data_real[15:0], mem_save};
				mem_data3 <= {data_imag, data_real[23:16]};
				rx_state <= sRx4;
			end
		end
		sRx4:		// write second and third dword
		begin
			mem_wen <= 1'd1;
			rx_state <= sRx5;
		end
		sRx5:
		begin
			mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			rx_state <= sRx6;
		end
		sRx6:
		begin
			mem_data <= mem_data3;
			rx_state <= sRx7;
		end
		sRx7:
		begin
			mem_wen <= 1'd1;
			rx_state <= sRx8;
		end
		sRx8:
		begin
			mem_wen <= 1'd0;
			rx_state <= sRx9;
		end
		sRx9:		// increment memory address
		begin
			if (mem_write_addr == MEM_DWORD_SIZE - 1)
			begin		// first block of memory is ready to be sent
				mem_write_addr <= mem_write_addr + 1'd1;
				rx_state <= sRx10;
			end
			else if (mem_write_addr == MEM_DWORD_SIZE * 2 - 1)
			begin		// second block of memory is ready to be sent
				mem_write_addr <= 1'd0;
				rx_state <= sRx11;
			end
			else
			begin
				mem_write_addr <= mem_write_addr + 1'd1;
				rx_state <= sRx0;
			end
		end
		sRx10:
		begin
			mem_block <= 1'd0;
			rx_state <= sRx0;
		end
		sRx11:
		begin
			mem_block <= 1'd1;
			rx_state <= sRx;
		end
		// HPSDR mode
	   hRx0:	   // control bytes prepare
		begin
			C0[7:3] <= C0_addr_cnt;
			C0[2:0] <= {dot, dash, ptt_hpsdr};
			case (C0_addr_cnt)
			0:
			begin 
				C1 <= {7'd0, clipping};
				C2 <= 8'h00;
				C3 <= 8'h00;
				C4 <= 8'd29;
			end
			1:
			begin
				C1 <= 8'h00;
				C2 <= 8'h00;
				C3 <= 8'h00;
				C4 <= 8'h00;
			end
			2:
			begin
				C1 <= 8'h00;
				C2 <= 8'h00;
				C3 <= 8'h00;
				C4 <= 8'h00;
			end
		   3:
			begin
				C1 <= 8'h00;
				C2 <= 8'h00;
				C3 <= 8'h00;
				C4 <= 8'h00;
			end
			4:
			begin
				C1 <= 8'h00;
				C2 <= 8'h00;
				C3 <= 8'h00;
				C4 <= 8'h00;
			end
		   endcase
			rx_state <= hRx1;
		end		
	hRx1:
		begin
		   if (C0_addr_cnt == 4) C0_addr_cnt <= 0;
			else C0_addr_cnt <= C0_addr_cnt + 1'd1;
		   mem_data <= {C0, 8'h7F, 8'h7F, 8'h7F}; //synchro bytes
			rx_state <= hRx2;
		end
		hRx2:
		begin
		   mem_wen <= 1'd1;
			rx_state <= hRx3;
		end
		hRx3:
		begin
		   mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			rx_state <= hRx4;	
		end
		hRx4:
      begin
		   mem_data <= {C4, C3, C2, C1};
		   rx_state <= hRx5;
      end		
		hRx5:
		begin
		   mem_wen <= 1'd1;
			rx_state <= hRx6;
		end
		hRx6:
		begin
		   mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			rx_state <= hRx7;	
		end
		hRx7:		// Wait for sample 
		begin
			if (data_avail)
			begin		// write first dword, save last two bytes LITTLE-ENDIAN
		      mem_data <= {data_real[23:16], data_imag[7:0], data_imag[15:8], data_imag[23:16]};  
				rx_state <= hRx8;
			end
		end
		hRx8:
		begin
			mem_wen <= 1'd1;
			rx_state <= hRx9;
		end
		hRx9:
		begin
			mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			rx_state <= hRx10;
		end
		hRx10:		
		begin
		   if (two_receiver_s) mem_data <= {data_imag_2[15:8], data_imag_2[23:16], data_real[7:0], data_real[15:8]};
			else mem_data <= {h_mic_data[7:0], h_mic_data[15:8], data_real[7:0], data_real[15:8]};
			rx_state <= hRx11;
		end
		hRx11:		
		begin
			mem_wen <= 1'd1;
			rx_state <= hRx12;
		end
		hRx12:
		begin
			mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			if (two_receiver_s) rx_state <= hRx13_1; else rx_state <= hRx13;
		end
		hRx13_1:
		begin
		  mem_data <= {data_real_2[7:0], data_real_2[15:8], data_real_2[23:16], data_imag_2[7:0]};
		  rx_state <= hRx13_2;
		end
		hRx13_2:
		begin
		   mem_wen <= 1'd1;
			rx_state <= hRx13_3;
		end
		hRx13_3:
		begin
		   mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			rx_state <= hRx13_4;
		end
		hRx13_4:
		begin
		   if (data_avail)
		   begin 
			   mem_data <= { data_imag[15:8], data_imag[23:16], h_mic_data[7:0], h_mic_data[15:8]};
				rx_state <= hRx13_5;
				end
		end
		hRx13_5:
		begin
		   mem_wen <= 1'd1;
			rx_state <= hRx13_6;
		end
		hRx13_6:
		begin
		   mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			rx_state <= hRx13_7;
		end
		hRx13_7:
		begin
		  mem_data <= {data_real[7:0], data_real[15:8], data_real[23:16], data_imag[7:0]};
		  rx_state <= hRx13_8;
		end
		hRx13_8:
		begin
		   mem_wen <= 1'd1;
			rx_state <= hRx13_9;
		end
		hRx13_9:
		begin
		   mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			rx_state <= hRx13_10;
		end
		hRx13_10:
		begin
		  mem_data <= {data_real_2[23:16], data_imag_2[7:0], data_imag_2[15:8], data_imag_2[23:16]};
		  rx_state <= hRx13_11;
		end
		hRx13_11:
		begin
		   mem_wen <= 1'd1;
			rx_state <= hRx13_12;
		end
		hRx13_12:
		begin
		   mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			rx_state <= hRx13_13;
		end
		hRx13_13:
		begin
		   mem_data <= {h_mic_data[7:0], h_mic_data[15:8], data_real_2[7:0], data_real_2[15:8]};
			rx_state <= hRx13_14;
		end
		hRx13_14:
		begin
		   mem_wen <= 1'd1;
			rx_state <= hRx13_15;
		end
		hRx13_15:
		begin
		   mem_wen <= 1'd0;
			mem_write_addr <= mem_write_addr + 1'd1;
			rx_state <= hRx13;
		end
		// end of cycle
		hRx13:
		begin
		   if (mem_write_addr == 128 || mem_write_addr == 384)  rx_state <= hRx0; // need second subframe
			else if (mem_write_addr == 256)  rx_state <= hRx14; // first block ready
			else if (mem_write_addr == 512)  rx_state <= hRx15;  // second block ready
			else  rx_state <= hRx7; // need odd samples
		end
		hRx14: //first block ready to sending
		begin
			mem_block <= 0;
		   rx_state <= hRx0; 
		end
		hRx15: // second frame ready to sending
		begin
			mem_block <= 1;
			mem_write_addr <= 0;
		   rx_state <= sRx; 
		end
		endcase
	end
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	always @(posedge clock)		// Rx2 state machine
	begin
		if (eth_nRESET == 0)	// we are in reset
		begin
			mem_write_addr_2 <= 1'd0;
			mem_wen_2 <= 0;
			rx_state_2 <= sRx_2;
		end
		else case (rx_state_2)
		// HiqSDR mode
		sRx_2:		// Wait for first sample of two samples
		begin
			if (data_avail_2)
			begin		// write first dword, save last two bytes LITTLE-ENDIAN
				mem_data_2 <= {data_imag_2[7:0], data_real_2};  
				mem_save_2 <= data_imag_2[23:8];	
				rx_state_2 <= sRx1_2;
			end
		end
		sRx1_2:
		begin
			mem_wen_2 <= 1'd1;
			rx_state_2 <= sRx2_2;
		end
		sRx2_2:
		begin
			mem_wen_2 <= 1'd0;
			mem_write_addr_2 <= mem_write_addr_2 + 1'd1;
			rx_state_2 <= sRx3_2;		// wait for next sample
		end
		sRx3_2:		// Wait for second sample of two samples
		begin
			if (data_avail_2)
			begin
				mem_data_2 <= {data_real_2[15:0], mem_save_2};
				mem_data3_2 <= {data_imag_2, data_real_2[23:16]};
				rx_state_2 <= sRx4_2;
			end
		end
		sRx4_2:		// write second and third dword
		begin
			mem_wen_2 <= 1'd1;
			rx_state_2 <= sRx5_2;
		end
		sRx5_2:
		begin
			mem_wen_2 <= 1'd0;
			mem_write_addr_2 <= mem_write_addr_2 + 1'd1;
			rx_state_2 <= sRx6_2;
		end
		sRx6_2:
		begin
			mem_data_2 <= mem_data3_2;
			rx_state_2 <= sRx7_2;
		end
		sRx7_2:
		begin
			mem_wen_2 <= 1'd1;
			rx_state_2 <= sRx8_2;
		end
		sRx8_2:
		begin
			mem_wen_2 <= 1'd0;
			rx_state_2 <= sRx9_2;
		end
		sRx9_2:		// increment memory address
		begin
			if (mem_write_addr_2 == MEM_DWORD_SIZE - 1)
			begin		// first block of memory is ready to be sent
				mem_write_addr_2 <= mem_write_addr_2 + 1'd1;
				rx_state_2 <= sRx10_2;
			end
			else if (mem_write_addr_2 == MEM_DWORD_SIZE * 2 - 1)
			begin		// second block of memory is ready to be sent
				mem_write_addr_2 <= 1'd0;
				rx_state_2 <= sRx11_2;
			end
			else
			begin
				mem_write_addr_2 <= mem_write_addr_2 + 1'd1;
				rx_state_2 <= sRx_2;
			end
		end
		sRx10_2:
		begin
			mem_block_2 <= 1'd0;
			rx_state_2 <= sRx_2;
		end
		sRx11_2:
		begin
			mem_block_2 <= 1'd1;
			rx_state_2 <= sRx_2;
		end
		endcase
	end

endmodule
	
