//
// David Fainitski
// for the project Odyssey
// 2016



	
module service (
   input clock, // 120 kHz
	input reset, 
	input ip_reset_req,
   output reg [79:0] srv_data,
	input [79:0] srv_data_sw,
	input srv_data_wr,
	output reg eth_reset,
	output reg srv_blink,
	// FLASH interface
	output reg DCLK,
	output reg DATAOUT,
	input      DATAIN,
	output reg FLASH_NCE
);


   initial 
	begin
	   DCLK <= 0;
		DATAOUT <= 0;
		FLASH_NCE <= 1;
	end
	
	parameter sSendCom   = 8'd50;
	parameter sSendCom1  = 8'd51;
	parameter sSendCom2  = 8'd52;
	parameter sSendCom3  = 8'd53;
	parameter sSendAddr  = 8'd60;
	parameter sSendAddr1 = 8'd61;
	parameter sSendAddr2 = 8'd62;
	parameter sSendAddr3 = 8'd63;
	parameter sReadSrv   = 8'd70;
	parameter sReadSrv1  = 8'd71;
	parameter sReadSrv2  = 8'd72;
	parameter sReadSts   = 8'd80;
	parameter sReadSts1  = 8'd81;
	parameter sReadSts2  = 8'd82;
	parameter sWriteSrv  = 8'd90;
	parameter sWriteSrv1 = 8'd91;
	parameter sWriteSrv2 = 8'd92;
	parameter sWriteSrv3 = 8'd93;
	
	reg [15:0] blink_cnt;
	reg [4:0] req_timer;
	reg [9:0] bit_cnt;
	reg [7:0] command, status;
	reg [23:0] address;
	reg srv_data_wr_old; 
	reg [7:0] state, return_state;
	reg ip_reset_flag;
   always @(posedge clock)
   begin
	   if (!reset)
		begin
	      eth_reset <= 0;
			srv_blink <= 0;
			blink_cnt <= 16'd60000; // 0.5 sec
			req_timer <= 5'd10;  // 5 * 0.5 = 5 sec
			srv_data <= 1'd0;
			ip_reset_flag <= 0;
			srv_data_wr_old <= 0;
			state <= 8'd0;
		end
	   else 
	   begin  // starting
	      case (state)
			0: if (ip_reset_req) // reset IP request
			   begin
				   if (blink_cnt != 0) blink_cnt <= blink_cnt - 1'd1;
				   else 
				   begin
					   blink_cnt <= 16'd60000;
				      srv_blink <= ~srv_blink;
					   req_timer <= req_timer - 1'd1;
					   state <= 5'd1;
				  end
				end
				else state <= 5'd2;
			1: if (req_timer == 0) // reset IP
			   begin
					ip_reset_flag <= 1;
					state <= 8'd2; 
				end
				else state <= 5'd0; // waiting
			2: begin     // read service's data from FLASH
			      //srv_blink <= 0;
					command <= 8'h03; // send the Read command
					return_state <= 8'd3;
					state <= sSendCom;
				end	
			3: begin
					address <= 24'h1F0000; // reading from 32-nd sector
					return_state <= 8'd4;
					state <= sSendAddr; // send the address
		      end	
			4: begin
			      return_state <= 8'd5; // read of 64 bytes service's data
					state <= sReadSrv;
		      end
		   5: begin   // finish, starting the ethernet
					if (ip_reset_flag)
					begin
					   srv_data[79 - 0*8 -: 8] <= 8'd192; // IP1
				      srv_data[79 - 1*8 -: 8] <= 8'd168; // IP2
                  srv_data[79 - 2*8 -: 8] <= 8'd2;   // IP3
					   srv_data[79 - 3*8 -: 8] <= 8'd160; // IP4
					   srv_data[79 - 4*8 -: 8] <= 8'h00;  // MAC1
					   srv_data[79 - 5*8 -: 8] <= 8'h0B;  // MAC2
					   srv_data[79 - 6*8 -: 8] <= 8'hB8;  // MAC3
					   srv_data[79 - 7*8 -: 8] <= 8'h60;  // MAC4
					   srv_data[79 - 8*8 -: 8] <= 8'h0C;  // MAC5
					   srv_data[79 - 9*8 -: 8] <= 8'h99;  // MAC6
						
						//
						ip_reset_flag <= 0;
						state <= 8'd5;
					end
					else 
					begin
			         eth_reset <= 1;
						if(srv_data_wr != srv_data_wr_old) //  waiting wor writing data
						begin
							srv_data_wr_old <= srv_data_wr;
							state <= 8'd6;
						end	
			         else state <= 8'd5; 
					end
	         end
			// writing service data	
			6: begin
			      command <= 8'h06; // write enable command
					return_state <= 8'd7;
					state <= sSendCom;
			   end
			7: begin
			      FLASH_NCE <= 1;
			      command <= 8'hD8;      //  erase a sector
					return_state <= 8'd8;
					state <= sSendCom;				
		      end	
			8: begin 
               address <= 24'h1F0000; // 32-nd sector
					return_state <= 8'd9;
					state <= sSendAddr;
		      end	
			9: begin              // waiting for the finish of erasing
			      FLASH_NCE <= 1;
					command <= 8'h05;  // read status command
					return_state <= 8'd10;
					state <= sSendCom; 
				end
	     10: begin
			      return_state <= 8'd11;
					state <= sReadSts;
	         end
	     11: begin
		         if (status[0] == 1) state <= 8'd9; // erase is finished ?
					else state <= 8'd12;           
		      end 
		  12: begin 
			      command <= 8'h06; // write enable command
					return_state <= 8'd13;
					state <= sSendCom;
			   end	
		  13: begin	
		         FLASH_NCE <= 1;
					command <= 8'h02;   // writing service's data
					return_state <= 8'd14;
					state <= sSendCom;
		      end 
		  14: begin
		         address <= 24'h1F0000; // write to first page of 32-nd sector 
					return_state <= 8'd15;
					state <= sSendAddr; 
		      end
		  15: begin
			      return_state <= 8'd16; // starting to write data
					state <= sWriteSrv;
		      end
		  16: begin  // waiting for the finish
				   command <= 8'h05;  // read status command
				   return_state <= 8'd17;
				   state <= sSendCom;
		      end
	     17: begin
			      return_state <= 8'd18;
					state <= sReadSts;
	         end
	     18: begin
		         if (status[0]) state <= 8'd16; 
					else begin srv_data <= srv_data_sw; state <= 8'd5; end  
		      end 		 
				
			
				
  sSendCom:	begin     // send command to FLASH
               bit_cnt <= 8'd7;
					FLASH_NCE <= 0;
					state <= sSendCom1;
            end
 sSendCom1:	begin
               DATAOUT <= command[bit_cnt]; 
					state <= sSendCom2;
				end
 sSendCom2:	begin
               DCLK <= 1;
				   state <= sSendCom3;
				end
 sSendCom3:	begin
               DCLK <= 0;
				   if (bit_cnt != 0) begin bit_cnt <= bit_cnt - 1'd1; state <= sSendCom1; end
				   else state <= return_state;
				end
 sSendAddr: begin
               bit_cnt <= 8'd23;
					state <= sSendAddr1;
            end
 sSendAddr1: begin
               DATAOUT <= address[bit_cnt];
					state <= sSendAddr2;
            end				
 sSendAddr2: begin
               DCLK <=1;
					state <= sSendAddr3;
            end
 sSendAddr3: begin
               DCLK <= 0;
					if (bit_cnt != 0) begin bit_cnt <= bit_cnt - 1'd1; state <= sSendAddr1; end
					else state <= return_state;
            end	
  sReadSrv: begin
               bit_cnt <= 79; // Read 64 bytes
				   state <= sReadSrv1;
            end 
 sReadSrv1:	begin
               srv_data[bit_cnt] <= DATAIN;
					DCLK <= 1;
               state <= sReadSrv2;
            end 
 sReadSrv2: begin
               DCLK <= 0;
					if (bit_cnt != 0) begin bit_cnt <= bit_cnt - 1'd1; state <= sReadSrv1; end
					else begin FLASH_NCE <= 1; state <= return_state; end
            end 
  sReadSts: begin
               bit_cnt <= 8'd7;
					state <= sReadSts1;
            end
 sReadSts1: begin
               status[bit_cnt] <= DATAIN;
				   DCLK <= 1;	
					state <= sReadSts2;
            end
 sReadSts2: begin
               DCLK <= 0;
					if (bit_cnt != 0) begin bit_cnt <= bit_cnt - 1'd1; state <= sReadSts1; end
					else begin FLASH_NCE <= 1; state <= return_state; end
            end
 sWriteSrv: begin
               bit_cnt <= 79; 
	            state <= sWriteSrv1;			  
            end 
sWriteSrv1: begin
               DATAOUT <= srv_data_sw[bit_cnt];
	            state <= sWriteSrv2;			  
            end 
sWriteSrv2: begin
               DCLK <= 1;
	            state <= sWriteSrv3;			  
            end
sWriteSrv3: begin
               DCLK <= 0;
					if (bit_cnt != 0) begin bit_cnt <= bit_cnt - 1'd1; state <= sWriteSrv1; end
	            else begin FLASH_NCE <= 1; state <= return_state; end			  
            end					
	default: state <= 1'd0;
			endcase
      end	
	end
	
	
	
	
endmodule	 